@@ -53,6 +53,13 @@ struct SDL_PrivateAudioData
 
 #define LIB_AAUDIO_SO "libaaudio.so"
 
+SDL_ELF_NOTE_DLOPEN(
+    "audio-aaudio",
+    "Support for audio through AAudio",
+    SDL_ELF_NOTE_DLOPEN_PRIORITY_SUGGESTED,
+    LIB_AAUDIO_SO
+);
+
 typedef struct AAUDIO_Data
 {
     SDL_SharedObject *handle;
@@ -65,11 +72,16 @@ static bool AAUDIO_LoadFunctions(AAUDIO_Data *data)
 {
 #define SDL_PROC(ret, func, params)                                                             \
     do {                                                                                        \
-        data->func = (ret (*) params)SDL_LoadFunction(data->handle, #func);                                     \
+        data->func = (ret (*) params)SDL_LoadFunction(data->handle, #func);                     \
         if (!data->func) {                                                                      \
             return SDL_SetError("Couldn't load AAUDIO function %s: %s", #func, SDL_GetError()); \
         }                                                                                       \
     } while (0);
+
+#define SDL_PROC_OPTIONAL(ret, func, params)                                                          \
+    do {                                                                                              \
+        data->func = (ret (*) params)SDL_LoadFunction(data->handle, #func);  /* if it fails, okay. */ \
+    } while (0);
 #include "SDL_aaudiofuncs.h"
     return true;
 }
@@ -308,8 +320,8 @@ static bool BuildAAudioStream(SDL_AudioDevice *device)
     ctx.AAudioStreamBuilder_setFormat(builder, format);
     ctx.AAudioStreamBuilder_setSampleRate(builder, device->spec.freq);
     ctx.AAudioStreamBuilder_setChannelCount(builder, device->spec.channels);
-	
-    // If no specific buffer size has been requested, the device will pick the optimal 
+
+    // If no specific buffer size has been requested, the device will pick the optimal
 	if(SDL_GetHint(SDL_HINT_AUDIO_DEVICE_SAMPLE_FRAMES)) {
 	    ctx.AAudioStreamBuilder_setBufferCapacityInFrames(builder, 2 * device->sample_frames); // AAudio requires that the buffer capacity is at least
 	    ctx.AAudioStreamBuilder_setFramesPerDataCallback(builder, device->sample_frames);      // twice the size of the data callback buffer size
@@ -327,6 +339,12 @@ static bool BuildAAudioStream(SDL_AudioDevice *device)
         SDL_Log("Low latency audio disabled");
     }
 
+    if (recording && ctx.AAudioStreamBuilder_setInputPreset) {    // optional API: requires Android 28
+        // try to use a microphone that is for recording external audio. Otherwise Android might choose the mic used for talking
+        // on the telephone when held to the user's ear, which is often not useful at any distance from the device.
+        ctx.AAudioStreamBuilder_setInputPreset(builder, AAUDIO_INPUT_PRESET_CAMCORDER);
+    }
+
     LOGI("AAudio Try to open %u hz %s %u channels samples %u",
          device->spec.freq, SDL_GetAudioFormatName(device->spec.format),
          device->spec.channels, device->sample_frames);
