@@ -42,6 +42,9 @@ typedef enum
     GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_NEAREST,
     GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_LINEAR,
     GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_PIXELART,
+    GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_NEAREST_COLORSWAP,
+    GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_LINEAR_COLORSWAP,
+    GLES2_SHADER_FRAGMENT_TEXTURE_PALETTE_PIXELART_COLORSWAP,
     GLES2_SHADER_FRAGMENT_TEXTURE_RGB,
     GLES2_SHADER_FRAGMENT_TEXTURE_RGB_PIXELART,
     GLES2_SHADER_FRAGMENT_TEXTURE_BGR,
