@@ -78,6 +78,9 @@ struct SDL_Surface
 
     /** info for fast blit mapping to other surfaces */
     SDL_BlitMap map;
+
+    /** Original pixels when RLE is enabled */
+    void *saved_pixels;
 };
 
 // Surface functions
@@ -89,5 +92,8 @@ extern float SDL_GetSurfaceSDRWhitePoint(SDL_Surface *surface, SDL_Colorspace co
 extern float SDL_GetDefaultHDRHeadroom(SDL_Colorspace colorspace);
 extern float SDL_GetSurfaceHDRHeadroom(SDL_Surface *surface, SDL_Colorspace colorspace);
 extern SDL_Surface *SDL_GetSurfaceImage(SDL_Surface *surface, float display_scale);
+extern SDL_Surface *SDL_ConvertSurfaceRect(SDL_Surface *surface, const SDL_Rect *rect, SDL_PixelFormat format);
+extern bool SDL_IsBMP(SDL_IOStream *src);
+extern bool SDL_IsPNG(SDL_IOStream *src);
 
 #endif // SDL_surface_c_h_
