@@ -48,9 +48,7 @@ typedef struct
 
 struct SDL_VideoData
 {
-    bool initializing;
     struct wl_display *display;
-    int display_disconnected;
     struct wl_registry *registry;
     struct wl_compositor *compositor;
     struct wl_shm *shm;
@@ -99,6 +97,8 @@ struct SDL_VideoData
     int output_count;
     int output_max;
 
+    bool initializing;
+    bool display_disconnected;
     bool display_externally_owned;
     bool scale_to_display_enabled;
 };
@@ -118,7 +118,6 @@ struct SDL_DisplayData
     SDL_DisplayOrientation orientation;
     int physical_width_mm, physical_height_mm;
     bool has_logical_position, has_logical_size;
-    bool running_colorspace_event_queue;
     SDL_HDROutputProperties HDR;
     SDL_DisplayID display;
     SDL_VideoDisplay placeholder;
@@ -141,6 +140,6 @@ struct wl_event_queue *Wayland_DisplayCreateQueue(struct wl_display *display, co
 
 extern bool Wayland_LoadLibdecor(SDL_VideoData *data, bool ignore_xdg);
 
-extern bool Wayland_VideoReconnect(SDL_VideoDevice *_this);
+extern bool Wayland_HandleDisplayDisconnected(SDL_VideoDevice *_this);
 
 #endif // SDL_waylandvideo_h_
