@@ -34,10 +34,14 @@
 
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
 static bool xinput2_initialized;
-
-#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
+#endif
+#if defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO) || defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH)
+static bool xinput2_scrolling_supported;
 static bool xinput2_multitouch_supported;
 #endif
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_GESTURE
+static bool xinput2_gesture_supported;
+#endif
 
 /* Opcode returned X11_XQueryExtension
  * It will be used in event processing
@@ -45,23 +49,72 @@ static bool xinput2_multitouch_supported;
  * this extension */
 static int xinput2_opcode;
 
-static void parse_valuators(const double *input_values, const unsigned char *mask, int mask_len,
-                            double *output_values, int output_values_len)
+static Atom xinput2_rel_x_atom;
+static Atom xinput2_rel_y_atom;
+static Atom xinput2_abs_x_atom;
+static Atom xinput2_abs_y_atom;
+
+// Pointer button remapping table
+static unsigned char *xinput2_pointer_button_map;
+static int xinput2_pointer_button_map_size;
+
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+typedef struct
 {
-    int i = 0, z = 0;
-    int top = mask_len * 8;
-    if (top > MAX_AXIS) {
-        top = MAX_AXIS;
+    int number;
+    int scroll_type;
+    double prev_value;
+    double increment;
+    bool prev_value_valid;
+} SDL_XInput2ScrollInfo;
+
+typedef struct
+{
+    int device_id;
+    int scroll_info_count;
+    SDL_XInput2ScrollInfo *scroll_info;
+} SDL_XInput2ScrollableDevice;
+
+static SDL_XInput2ScrollableDevice *scrollable_devices;
+static int scrollable_device_count;
+#endif
+
+static void parse_relative_valuators(SDL_XInput2DeviceInfo *devinfo, const XIRawEvent *rawev)
+{
+    SDL_Mouse *mouse = SDL_GetMouse();
+    double processed_coords[2] = { 0.0, 0.0 };
+    int values_i = 0, found = 0;
+
+    // Use the raw values if a custom transform function is set, or the relative system scale hint is unset.
+    const bool use_raw_vals = mouse->InputTransform || !mouse->enable_relative_system_scale;
+
+    for (int i = 0; i < rawev->valuators.mask_len * 8 && found < 2; ++i) {
+        if (!XIMaskIsSet(rawev->valuators.mask, i)) {
+            continue;
+        }
+
+        for (int j = 0; j < 2; ++j) {
+            if (devinfo->number[j] == i) {
+                const double current_val = use_raw_vals ? rawev->raw_values[values_i] : rawev->valuators.values[values_i];
+
+                if (devinfo->relative[j]) {
+                    processed_coords[j] = current_val;
+                } else {
+                    processed_coords[j] = (current_val - devinfo->prev_coords[j]); // convert absolute to relative
+                    devinfo->prev_coords[j] = current_val;
+                }
+                ++found;
+
+                break;
+            }
+        }
+
+        ++values_i;
     }
 
-    SDL_memset(output_values, 0, output_values_len * sizeof(double));
-    for (; i < top && z < output_values_len; i++) {
-        if (XIMaskIsSet(mask, i)) {
-            const int value = (int)*input_values;
-            output_values[z] = value;
-            input_values++;
-        }
-        z++;
+    // Relative mouse motion is delivered to the window with keyboard focus
+    if (mouse->relative_mode && SDL_GetKeyboardFocus()) {
+        SDL_SendMouseMotion(rawev->time, mouse->focus, (SDL_MouseID)rawev->sourceid, true, (float)processed_coords[0], (float)processed_coords[1]);
     }
 }
 
@@ -77,6 +130,7 @@ static bool xinput2_version_atleast(const int version, const int wantmajor, cons
     return version >= ((wantmajor * 1000) + wantminor);
 }
 
+// !!! FIXME: isn't this just X11_FindWindow?
 static SDL_WindowData *xinput2_get_sdlwindowdata(SDL_VideoData *videodata, Window window)
 {
     int i;
@@ -95,6 +149,62 @@ static SDL_Window *xinput2_get_sdlwindow(SDL_VideoData *videodata, Window window
     return windowdata ? windowdata->window : NULL;
 }
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+static void xinput2_reset_scrollable_valuators(SDL_VideoData *videodata)
+{
+    for (int i = 0; i < scrollable_device_count; ++i) {
+        for (int j = 0; j < scrollable_devices[i].scroll_info_count; ++j) {
+            scrollable_devices[i].scroll_info[j].prev_value_valid = false;
+        }
+    }
+}
+
+static void xinput2_parse_scrollable_valuators(const XIDeviceEvent *xev)
+{
+    for (int i = 0; i < scrollable_device_count; ++i) {
+        const SDL_XInput2ScrollableDevice *sd = &scrollable_devices[i];
+        if (xev->sourceid == sd->device_id) {
+            int values_i = 0;
+            for (int j = 0; j < xev->valuators.mask_len * 8; ++j) {
+                if (!XIMaskIsSet(xev->valuators.mask, j)) {
+                    continue;
+                }
+
+                for (int k = 0; k < sd->scroll_info_count; ++k) {
+                    SDL_XInput2ScrollInfo *info = &sd->scroll_info[k];
+                    if (info->number == j) {
+                        const double current_val = xev->valuators.values[values_i];
+                        const double delta = (info->prev_value - current_val) / info->increment;
+                        /* Ignore very large jumps that can happen as a result of overflowing
+                         * the maximum range, as the driver will reset the position to zero
+                         * at "something that's close to 2^32".
+                         *
+                         * The first scroll event is meaningless by itself and must be discarded,
+                         * as it is only useful for establishing a baseline for future deltas.
+                         * This is a known deficiency of the XInput2 scroll protocol, and,
+                         * unfortunately, there is nothing we can do about it.
+                         *
+                         * http://who-t.blogspot.com/2012/06/xi-21-protocol-design-issues.html
+                         */
+                        if (info->prev_value_valid && SDL_fabs(delta) < (double)SDL_MAX_SINT32 * 0.95) {
+                            const double x = info->scroll_type == XIScrollTypeHorizontal ? delta : 0;
+                            const double y = info->scroll_type == XIScrollTypeVertical ? delta : 0;
+
+                            SDL_Mouse *mouse = SDL_GetMouse();
+                            SDL_SendMouseWheel(xev->time, mouse->focus, (SDL_MouseID)xev->sourceid, (float)x, (float)y, SDL_MOUSEWHEEL_NORMAL);
+                        }
+                        info->prev_value = current_val;
+                        info->prev_value_valid = true;
+                    }
+                }
+
+                ++values_i;
+            }
+        }
+    }
+}
+#endif // SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
 static void xinput2_normalize_touch_coordinates(SDL_Window *window, double in_x, double in_y, float *out_x, float *out_y)
 {
@@ -117,8 +227,6 @@ static void xinput2_normalize_touch_coordinates(SDL_Window *window, double in_x,
 }
 #endif // SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
 
-#endif // SDL_VIDEO_DRIVER_X11_XINPUT2
-
 bool X11_InitXinput2(SDL_VideoDevice *_this)
 {
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
@@ -126,7 +234,7 @@ bool X11_InitXinput2(SDL_VideoDevice *_this)
 
     int version = 0;
     XIEventMask eventmask;
-    unsigned char mask[4] = { 0, 0, 0, 0 };
+    unsigned char mask[5] = { 0, 0, 0, 0, 0 };
     int event, err;
 
     /* XInput2 is required for relative mouse mode, so you probably want to leave this enabled */
@@ -148,17 +256,29 @@ bool X11_InitXinput2(SDL_VideoDevice *_this)
         return false; // X server does not have XInput at all
     }
 
-    // We need at least 2.2 for Multitouch, 2.0 otherwise.
-    version = query_xinput2_version(data->display, 2, 2);
+    // We need at least 2.4 for Gesture, 2.2 for Multitouch, 2.0 otherwise.
+    version = query_xinput2_version(data->display, 2, 4);
     if (!xinput2_version_atleast(version, 2, 0)) {
         return false; // X server does not support the version we want at all.
     }
 
     xinput2_initialized = true;
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO // Smooth scrolling needs XInput 2.1
+    xinput2_scrolling_supported = xinput2_version_atleast(version, 2, 1);
+#endif
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH // Multitouch needs XInput 2.2
     xinput2_multitouch_supported = xinput2_version_atleast(version, 2, 2);
 #endif
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_GESTURE // Gesture needs XInput 2.4
+    xinput2_gesture_supported = xinput2_version_atleast(version, 2, 4);
+#endif
+
+    // Populate the atoms for finding relative axes
+    xinput2_rel_x_atom = X11_XInternAtom(data->display, "Rel X", False);
+    xinput2_rel_y_atom = X11_XInternAtom(data->display, "Rel Y", False);
+    xinput2_abs_x_atom = X11_XInternAtom(data->display, "Abs X", False);
+    xinput2_abs_y_atom = X11_XInternAtom(data->display, "Abs Y", False);
 
     // Enable raw motion events for this display
     SDL_zero(eventmask);
@@ -171,9 +291,15 @@ bool X11_InitXinput2(SDL_VideoDevice *_this)
     XISetMask(mask, XI_RawButtonPress);
     XISetMask(mask, XI_RawButtonRelease);
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+    if (xinput2_scrolling_supported) {
+        XISetMask(mask, XI_Motion);
+    }
+#endif
+
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
     // Enable raw touch events if supported
-    if (X11_Xinput2IsMultitouchSupported()) {
+    if (xinput2_multitouch_supported) {
         XISetMask(mask, XI_RawTouchBegin);
         XISetMask(mask, XI_RawTouchUpdate);
         XISetMask(mask, XI_RawTouchEnd);
@@ -191,7 +317,8 @@ bool X11_InitXinput2(SDL_VideoDevice *_this)
     XISetMask(mask, XI_HierarchyChanged);
     X11_XISelectEvents(data->display, DefaultRootWindow(data->display), &eventmask, 1);
 
-    X11_Xinput2UpdateDevices(_this, true);
+    X11_Xinput2UpdateDevices(_this);
+    X11_Xinput2UpdatePointerMapping(_this);
 
     return true;
 #else
@@ -199,6 +326,47 @@ bool X11_InitXinput2(SDL_VideoDevice *_this)
 #endif
 }
 
+void X11_QuitXinput2(SDL_VideoDevice *_this)
+{
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
+    SDL_free(xinput2_pointer_button_map);
+    xinput2_pointer_button_map = NULL;
+    xinput2_pointer_button_map_size = 0;
+
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+    for (int i = 0; i < scrollable_device_count; ++i) {
+        SDL_free(scrollable_devices[i].scroll_info);
+    }
+    SDL_free(scrollable_devices);
+    scrollable_devices = NULL;
+    scrollable_device_count = 0;
+#endif
+#endif
+}
+
+void X11_Xinput2UpdatePointerMapping(SDL_VideoDevice *_this)
+{
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
+    if (X11_Xinput2IsInitialized()) {
+        SDL_VideoData *vid = _this->internal;
+
+        SDL_free(xinput2_pointer_button_map);
+        xinput2_pointer_button_map = NULL;
+        xinput2_pointer_button_map_size = 0;
+
+        xinput2_pointer_button_map_size = X11_XGetPointerMapping(vid->display, NULL, 0);
+        if (xinput2_pointer_button_map_size) {
+            xinput2_pointer_button_map = SDL_calloc(xinput2_pointer_button_map_size, sizeof(unsigned char));
+            if (xinput2_pointer_button_map) {
+                xinput2_pointer_button_map_size = X11_XGetPointerMapping(vid->display, xinput2_pointer_button_map, xinput2_pointer_button_map_size);
+            } else {
+                xinput2_pointer_button_map_size = 0;
+            }
+        }
+    }
+#endif
+}
+
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
 // xi2 device went away? take it out of the list.
 static void xinput2_remove_device_info(SDL_VideoData *videodata, const int device_id)
@@ -227,7 +395,6 @@ static SDL_XInput2DeviceInfo *xinput2_get_device_info(SDL_VideoData *videodata,
     SDL_XInput2DeviceInfo *prev = NULL;
     SDL_XInput2DeviceInfo *devinfo;
     XIDeviceInfo *xidevinfo;
-    int axis = 0;
     int i;
 
     for (devinfo = videodata->mouse_device_info; devinfo; devinfo = devinfo->next) {
@@ -257,18 +424,49 @@ static SDL_XInput2DeviceInfo *xinput2_get_device_info(SDL_VideoData *videodata,
 
     devinfo->device_id = device_id;
 
-    /* !!! FIXME: this is sort of hacky because we only care about the first two axes we see, but any given
-       !!! FIXME:  axis could be relative or absolute, and they might not even be the X and Y axes!
-       !!! FIXME:  But we go on, for now. Maybe we need a more robust mouse API in SDL3... */
+    /* Search for relative axes with the following priority:
+     *  - Labelled 'Rel X'/'Rel Y'
+     *   - Labelled 'Abs X'/'Abs Y'
+     *    - The first two axes found
+     */
+    bool have_rel_x = false, have_rel_y = false;
+    bool have_abs_x = false, have_abs_y = false;
+    int axis_index = 0;
     for (i = 0; i < xidevinfo->num_classes; i++) {
         const XIValuatorClassInfo *v = (const XIValuatorClassInfo *)xidevinfo->classes[i];
         if (v->type == XIValuatorClass) {
-            devinfo->relative[axis] = (v->mode == XIModeRelative);
-            devinfo->minval[axis] = v->min;
-            devinfo->maxval[axis] = v->max;
-            if (++axis >= 2) {
+            if (v->label == xinput2_rel_x_atom || (v->label == xinput2_abs_x_atom && !have_rel_x) ||
+                (axis_index == 0 && !have_rel_x && !have_abs_x)) {
+                devinfo->number[0] = v->number;
+                devinfo->relative[0] = (v->mode == XIModeRelative);
+                devinfo->minval[0] = v->min;
+                devinfo->maxval[0] = v->max;
+
+                if (v->label == xinput2_rel_x_atom) {
+                    have_rel_x = true;
+                } else if (v->label == xinput2_abs_x_atom) {
+                    have_abs_x = true;
+                }
+            } else if (v->label == xinput2_rel_y_atom || (v->label == xinput2_abs_y_atom && !have_rel_y) ||
+                       (axis_index == 1 && !have_rel_y && !have_abs_y)) {
+                devinfo->number[1] = v->number;
+                devinfo->relative[1] = (v->mode == XIModeRelative);
+                devinfo->minval[1] = v->min;
+                devinfo->maxval[1] = v->max;
+
+                if (v->label == xinput2_rel_y_atom) {
+                    have_rel_y = true;
+                } else if (v->label == xinput2_abs_y_atom) {
+                    have_abs_y = true;
+                }
+            }
+
+            // If two relative axes were found, nothing more to do.
+            if (have_rel_x && have_rel_y) {
                 break;
             }
+
+            ++axis_index;
         }
     }
 
@@ -315,45 +513,33 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
     //case XI_PropertyEvent:
     //case XI_DeviceChanged:
 
+    case XI_PropertyEvent:
+    {
+        const XIPropertyEvent *proev = (const XIPropertyEvent *)cookie->data;
+        // Handle pen proximity enter/leave
+        if (proev->what == XIPropertyModified && proev->property == videodata->atoms.pen_atom_wacom_serial_ids) {
+            const XIDeviceEvent *xev = (const XIDeviceEvent *)cookie->data;
+            SDL_WindowData *windowdata = X11_FindWindow(_this, xev->event);
+            X11_NotifyPenProximityChange(_this, windowdata ? windowdata->window : NULL, proev->deviceid);
+        }
+    } break;
+
     case XI_RawMotion:
     {
         const XIRawEvent *rawev = (const XIRawEvent *)cookie->data;
         const bool is_pen = X11_FindPenByDeviceID(rawev->sourceid) != NULL;
-        SDL_Mouse *mouse = SDL_GetMouse();
-        SDL_XInput2DeviceInfo *devinfo;
-        double coords[2];
-        double processed_coords[2];
-        int i;
-        Uint64 timestamp = X11_GetEventTimestamp(rawev->time);
 
         videodata->global_mouse_changed = true;
         if (is_pen) {
             break; // Pens check for XI_Motion instead
         }
 
-        devinfo = xinput2_get_device_info(videodata, rawev->deviceid);
+        SDL_XInput2DeviceInfo *devinfo = xinput2_get_device_info(videodata, rawev->deviceid);
         if (!devinfo) {
             break; // oh well.
         }
 
-        parse_valuators(rawev->raw_values, rawev->valuators.mask,
-                        rawev->valuators.mask_len, coords, 2);
-
-        for (i = 0; i < 2; i++) {
-            if (devinfo->relative[i]) {
-                processed_coords[i] = coords[i];
-            } else {
-                processed_coords[i] = devinfo->prev_coords[i] - coords[i]; // convert absolute to relative
-            }
-        }
-
-        // Relative mouse motion is delivered to the window with keyboard focus
-        if (mouse->relative_mode && SDL_GetKeyboardFocus()) {
-            SDL_SendMouseMotion(timestamp, mouse->focus, (SDL_MouseID)rawev->sourceid, true, (float)processed_coords[0], (float)processed_coords[1]);
-        }
-
-        devinfo->prev_coords[0] = coords[0];
-        devinfo->prev_coords[1] = coords[1];
+        parse_relative_valuators(devinfo, rawev);
     } break;
 
     case XI_KeyPress:
@@ -407,8 +593,13 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
     {
         const XIDeviceEvent *xev = (const XIDeviceEvent *)cookie->data;
         X11_PenHandle *pen = X11_FindPenByDeviceID(xev->sourceid);
-        const int button = xev->detail;
+        int button = xev->detail;
         const bool down = (cookie->evtype == XI_ButtonPress);
+#if defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO) || defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH)
+        bool pointer_emulated = (xev->flags & XIPointerEmulated) != 0;
+#else
+        bool pointer_emulated = false;
+#endif
 
         if (pen) {
             if (xev->deviceid != xev->sourceid) {
@@ -422,11 +613,18 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
             } else {
                 SDL_SendPenButton(0, pen->pen, window, button - 1, down);
             }
-        } else {
+        } else if (!pointer_emulated) {
             // Otherwise assume a regular mouse
             SDL_WindowData *windowdata = xinput2_get_sdlwindowdata(videodata, xev->event);
             int x_ticks = 0, y_ticks = 0;
 
+            // Slave pointer devices don't have button remapping applied automatically, so do it manually.
+            if (xev->deviceid != videodata->xinput_master_pointer_device) {
+                if (button <= xinput2_pointer_button_map_size) {
+                    button = xinput2_pointer_button_map[button - 1];
+                }
+            }
+
             /* Discard wheel events from "Master" devices to avoid duplicates,
              * as coarse wheel events are stateless and can't be deduplicated.
              */
@@ -443,13 +641,19 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
         }
     } break;
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+    case XI_Enter:
+        xinput2_reset_scrollable_valuators(videodata);
+        break;
+#endif
+
     /* Register to receive XI_Motion (which deactivates MotionNotify), so that we can distinguish
        real mouse motions from synthetic ones, for multitouch and pen support. */
     case XI_Motion:
     {
         const XIDeviceEvent *xev = (const XIDeviceEvent *)cookie->data;
-#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
-        bool pointer_emulated = ((xev->flags & XIPointerEmulated) != 0);
+#if defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO) || defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH)
+        bool pointer_emulated = (xev->flags & XIPointerEmulated) != 0;
 #else
         bool pointer_emulated = false;
 #endif
@@ -475,14 +679,22 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
                     SDL_SendPenAxis(0, pen->pen, window, (SDL_PenAxis) i, axes[i]);
                 }
             }
-        } else if (!pointer_emulated && xev->deviceid == videodata->xinput_master_pointer_device) {
-            // Use the master device for non-relative motion, as the slave devices can seemingly lag behind.
-            SDL_Mouse *mouse = SDL_GetMouse();
-            if (!mouse->relative_mode) {
-                SDL_Window *window = xinput2_get_sdlwindow(videodata, xev->event);
-                if (window) {
-                    X11_ProcessHitTest(_this, window->internal, (float)xev->event_x, (float)xev->event_y, false);
-                    SDL_SendMouseMotion(0, window, SDL_GLOBAL_MOUSE_ID, false, (float)xev->event_x, (float)xev->event_y);
+        } else if (!pointer_emulated) {
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+            if (xev->deviceid == xev->sourceid) {
+                xinput2_parse_scrollable_valuators(xev);
+            }
+#endif
+
+            if (xev->deviceid == videodata->xinput_master_pointer_device) {
+                // Use the master device for non-relative motion, as the slave devices can seemingly lag behind.
+                SDL_Mouse *mouse = SDL_GetMouse();
+                if (!mouse->relative_mode) {
+                    SDL_Window *window = xinput2_get_sdlwindow(videodata, xev->event);
+                    if (window) {
+                        X11_ProcessHitTest(_this, window->internal, (float)xev->event_x, (float)xev->event_y, false);
+                        SDL_SendMouseMotion(0, window, SDL_GLOBAL_MOUSE_ID, false, (float)xev->event_x, (float)xev->event_y);
+                    }
                 }
             }
         }
@@ -516,6 +728,28 @@ void X11_HandleXinput2Event(SDL_VideoDevice *_this, XGenericEventCookie *cookie)
         SDL_SendTouchMotion(0, xev->sourceid, xev->detail, window, x, y, 1.0);
     } break;
 #endif // SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
+
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_GESTURE
+    case XI_GesturePinchBegin:
+    case XI_GesturePinchUpdate:
+    case XI_GesturePinchEnd:
+    {
+        const XIGesturePinchEvent *xev = (const XIGesturePinchEvent *)cookie->data;
+        float x, y;
+        SDL_Window *window = xinput2_get_sdlwindow(videodata, xev->event);
+        xinput2_normalize_touch_coordinates(window, xev->event_x, xev->event_y, &x, &y);
+
+        if (cookie->evtype == XI_GesturePinchBegin) {
+            SDL_SendPinch(SDL_EVENT_PINCH_BEGIN, 0, window, 0);
+        } else if (cookie->evtype == XI_GesturePinchUpdate) {
+            SDL_SendPinch(SDL_EVENT_PINCH_UPDATE, 0, window, (float)xev->scale);
+        } else {
+            SDL_SendPinch(SDL_EVENT_PINCH_END, 0, window, 0);
+        }
+    } break;
+
+#endif // SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_GESTURE
+
     }
 #endif // SDL_VIDEO_DRIVER_X11_XINPUT2
 }
@@ -524,29 +758,44 @@ void X11_InitXinput2Multitouch(SDL_VideoDevice *_this)
 {
 }
 
-void X11_Xinput2SelectTouch(SDL_VideoDevice *_this, SDL_Window *window)
+void X11_Xinput2Select(SDL_VideoDevice *_this, SDL_Window *window)
 {
-#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
-    SDL_VideoData *data = NULL;
+#if defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO) || defined(SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH)
+    SDL_VideoData *data = _this->internal;
+    SDL_WindowData *window_data = window->internal;
     XIEventMask eventmask;
-    unsigned char mask[4] = { 0, 0, 0, 0 };
-    SDL_WindowData *window_data = NULL;
+    unsigned char mask[5] = { 0, 0, 0, 0, 0 };
 
-    if (!X11_Xinput2IsMultitouchSupported()) {
+    if (!xinput2_scrolling_supported && !xinput2_multitouch_supported) {
         return;
     }
 
-    data = _this->internal;
-    window_data = window->internal;
-
     eventmask.deviceid = XIAllMasterDevices;
     eventmask.mask_len = sizeof(mask);
     eventmask.mask = mask;
 
-    XISetMask(mask, XI_TouchBegin);
-    XISetMask(mask, XI_TouchUpdate);
-    XISetMask(mask, XI_TouchEnd);
-    XISetMask(mask, XI_Motion);
+    if (xinput2_scrolling_supported) {
+        /* Track enter events that inform us that we need to update
+         * the previous scroll coordinates since we cannot track
+         * them outside our window.
+         */
+        XISetMask(mask, XI_Enter);
+    }
+
+    if (xinput2_multitouch_supported) {
+        XISetMask(mask, XI_TouchBegin);
+        XISetMask(mask, XI_TouchUpdate);
+        XISetMask(mask, XI_TouchEnd);
+        XISetMask(mask, XI_Motion);
+    }
+
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_GESTURE
+    if (xinput2_gesture_supported) {
+        XISetMask(mask, XI_GesturePinchBegin);
+        XISetMask(mask, XI_GesturePinchUpdate);
+        XISetMask(mask, XI_GesturePinchEnd);
+    }
+#endif
 
     X11_XISelectEvents(data->display, window_data->xwindow, &eventmask, 1);
 #endif
@@ -610,15 +859,6 @@ bool X11_Xinput2SelectMouseAndKeyboard(SDL_VideoDevice *_this, SDL_Window *windo
     return false;
 }
 
-bool X11_Xinput2IsMultitouchSupported(void)
-{
-#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
-    return xinput2_initialized && xinput2_multitouch_supported;
-#else
-    return true;
-#endif
-}
-
 void X11_Xinput2GrabTouch(SDL_VideoDevice *_this, SDL_Window *window)
 {
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
@@ -629,7 +869,7 @@ void X11_Xinput2GrabTouch(SDL_VideoDevice *_this, SDL_Window *window)
     XIGrabModifiers mods;
     XIEventMask eventmask;
 
-    if (!X11_Xinput2IsMultitouchSupported()) {
+    if (!xinput2_multitouch_supported) {
         return;
     }
 
@@ -657,7 +897,7 @@ void X11_Xinput2UngrabTouch(SDL_VideoDevice *_this, SDL_Window *window)
 
     XIGrabModifiers mods;
 
-    if (!X11_Xinput2IsMultitouchSupported()) {
+    if (!xinput2_multitouch_supported) {
         return;
     }
 
@@ -720,7 +960,7 @@ static bool HasDeviceID64(Uint64 deviceID, const Uint64 *list, int count)
 
 #endif // SDL_VIDEO_DRIVER_X11_XINPUT2
 
-void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
+void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this)
 {
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2
     SDL_VideoData *data = _this->internal;
@@ -738,7 +978,6 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
     Uint64 *old_touch_devices = NULL;
     int new_touch_count = 0;
     Uint64 *new_touch_devices = NULL;
-    bool send_event = !initial_check;
 
     SDL_assert(X11_Xinput2IsInitialized());
 
@@ -748,6 +987,16 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
     old_mice = SDL_GetMice(&old_mouse_count);
     old_touch_devices = SDL_GetTouchDevices(&old_touch_count);
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+    // Scroll devices don't get add/remove events, so just rebuild the list.
+    for (int i = 0; i < scrollable_device_count; ++i) {
+        SDL_free(scrollable_devices[i].scroll_info);
+    }
+    SDL_free(scrollable_devices);
+    scrollable_devices = NULL;
+    scrollable_device_count = 0;
+#endif
+
     for (int i = 0; i < ndevices; i++) {
         XIDeviceInfo *dev = &info[i];
 
@@ -758,7 +1007,7 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
                 SDL_KeyboardID keyboardID = (SDL_KeyboardID)dev->deviceid;
                 AddDeviceID(keyboardID, &new_keyboards, &new_keyboard_count);
                 if (!HasDeviceID(keyboardID, old_keyboards, old_keyboard_count)) {
-                    SDL_AddKeyboard(keyboardID, dev->name, send_event);
+                    SDL_AddKeyboard(keyboardID, dev->name);
                 }
             }
             break;
@@ -770,7 +1019,7 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
                 SDL_MouseID mouseID = (SDL_MouseID)dev->deviceid;
                 AddDeviceID(mouseID, &new_mice, &new_mouse_count);
                 if (!HasDeviceID(mouseID, old_mice, old_mouse_count)) {
-                    SDL_AddMouse(mouseID, dev->name, send_event);
+                    SDL_AddMouse(mouseID, dev->name);
                 }
             }
             break;
@@ -778,6 +1027,54 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
             break;
         }
 
+#ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_SCROLLINFO
+        SDL_XInput2ScrollableDevice *sd = NULL;
+        int allocated_scroll_info_count = 0;
+
+        for (int j = 0; j < dev->num_classes; j++) {
+            const XIAnyClassInfo *class = dev->classes[j];
+            const XIScrollClassInfo *s = (XIScrollClassInfo *)class;
+
+            if (class->type != XIScrollClass) {
+                continue;
+            }
+
+            // Allocate a new scrollable device.
+            if (!sd) {
+                scrollable_devices = SDL_realloc(scrollable_devices, (scrollable_device_count + 1) * sizeof(SDL_XInput2ScrollableDevice));
+                if (!scrollable_devices) {
+                    // No memory; so just skip this.
+                    break;
+                }
+
+                sd = &scrollable_devices[scrollable_device_count];
+                ++scrollable_device_count;
+
+                SDL_zerop(sd);
+                sd->device_id = dev->deviceid;
+            }
+
+            // Allocate new scroll info entries two at a time, as they typically come in a horizontal/vertical pair.
+            if (sd->scroll_info_count == allocated_scroll_info_count) {
+                sd->scroll_info = SDL_realloc(sd->scroll_info, (allocated_scroll_info_count + 2) * sizeof(SDL_XInput2ScrollInfo));
+                if (!sd->scroll_info) {
+                    // No memory; just skip this.
+                    break;
+                }
+
+                allocated_scroll_info_count += 2;
+            }
+
+            SDL_XInput2ScrollInfo *scroll_info = &sd->scroll_info[sd->scroll_info_count];
+            ++sd->scroll_info_count;
+
+            SDL_zerop(scroll_info);
+            scroll_info->number = s->number;
+            scroll_info->scroll_type = s->scroll_type;
+            scroll_info->increment = s->increment;
+        }
+#endif
+
 #ifdef SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH
         for (int j = 0; j < dev->num_classes; j++) {
             Uint64 touchID;
@@ -806,13 +1103,13 @@ void X11_Xinput2UpdateDevices(SDL_VideoDevice *_this, bool initial_check)
 
     for (int i = old_keyboard_count; i--;) {
         if (!HasDeviceID(old_keyboards[i], new_keyboards, new_keyboard_count)) {
-            SDL_RemoveKeyboard(old_keyboards[i], send_event);
+            SDL_RemoveKeyboard(old_keyboards[i]);
         }
     }
 
     for (int i = old_mouse_count; i--;) {
         if (!HasDeviceID(old_mice[i], new_mice, new_mouse_count)) {
-            SDL_RemoveMouse(old_mice[i], send_event);
+            SDL_RemoveMouse(old_mice[i]);
         }
     }
 
