@@ -21,7 +21,7 @@ static SDL_Renderer *renderer = NULL;
 
 static const char *mime_types[] = {
     "text/plain",
-    "image/bmp",
+    "image/png",
 };
 
 static const void *ClipboardDataCallback(void *userdata, const char *mime_type, size_t *size)
@@ -30,9 +30,9 @@ static const void *ClipboardDataCallback(void *userdata, const char *mime_type,
         const char *text = "Hello world!";
         *size = SDL_strlen(text);
         return text;
-    } else if (SDL_strcmp(mime_type, "image/bmp") == 0) {
-        *size = icon_bmp_len;
-        return icon_bmp;
+    } else if (SDL_strcmp(mime_type, "image/png") == 0) {
+        *size = icon_png_len;
+        return icon_png;
     } else {
         SDL_Log("Called with unexpected mime type: %s", mime_type);
         return NULL;
@@ -46,7 +46,7 @@ SDL_AppResult SDL_AppInit(void **appstate, int argc, char *argv[])
         return SDL_APP_FAILURE;
     }
 
-    if (!SDL_CreateWindowAndRenderer("testclipboard", 640, 480, 0, &window, &renderer)) {
+    if (!SDL_CreateWindowAndRenderer("testclipboard", 640, 480, SDL_WINDOW_RESIZABLE, &window, &renderer)) {
         SDL_Log("Couldn't create window and renderer: %s", SDL_GetError());
         return SDL_APP_FAILURE;
     }
@@ -93,9 +93,23 @@ SDL_AppResult SDL_AppEvent(void *appstate, SDL_Event *event)
 
 static float PrintClipboardText(float x, float y, const char *mime_type)
 {
-    void *data = SDL_GetClipboardData(mime_type, NULL);
+    size_t size;
+    void *data = SDL_GetClipboardData(mime_type, &size);
     if (data) {
-        SDL_RenderDebugText(renderer, x, y, (const char *)data);
+        char *text = (char *)data;
+        if (size > 2 && text[2] == '\0') {
+            /* UCS-4 data */
+            text = (char *)SDL_iconv_string("UTF-8", "UCS-4", data, size+4);
+        } else if (size > 1 && text[1] == '\0') {
+            /* UCS-2 data */
+            text = (char *)SDL_iconv_string("UTF-8", "UCS-2", data, size+2);
+        }
+        if (text) {
+            SDL_RenderDebugText(renderer, x, y, text);
+        }
+        if (text != data) {
+            SDL_free(text);
+        }
         SDL_free(data);
         return SDL_DEBUG_TEXT_FONT_CHARACTER_SIZE + 2.0f;
     }
@@ -114,34 +128,32 @@ static float PrintPrimarySelectionText(float x, float y)
 static float PrintClipboardImage(float x, float y, const char *mime_type)
 {
     /* We don't actually need to read this data each frame, but this is a simple example */
-    if (SDL_strcmp(mime_type, "image/bmp") == 0) {
-        size_t size;
-        void *data = SDL_GetClipboardData(mime_type, &size);
-        if (data) {
-            float w = 0.0f, h = 0.0f;
-            bool rendered = false;
-            SDL_IOStream *stream = SDL_IOFromConstMem(data, size);
-            if (stream) {
-                SDL_Surface *surface = SDL_LoadBMP_IO(stream, false);
-                if (surface) {
-                    SDL_Texture *texture = SDL_CreateTextureFromSurface(renderer, surface);
-                    if (texture) {
-                        SDL_GetTextureSize(texture, &w, &h);
+    size_t size;
+    void *data = SDL_GetClipboardData(mime_type, &size);
+    if (data) {
+        float w = 0.0f, h = 0.0f;
+        bool rendered = false;
+        SDL_IOStream *stream = SDL_IOFromConstMem(data, size);
+        if (stream) {
+            SDL_Surface *surface = SDL_LoadSurface_IO(stream, false);
+            if (surface) {
+                SDL_Texture *texture = SDL_CreateTextureFromSurface(renderer, surface);
+                if (texture) {
+                    SDL_GetTextureSize(texture, &w, &h);
 
-                        SDL_FRect dst = { x, y, w, h };
-                        rendered = SDL_RenderTexture(renderer, texture, NULL, &dst);
-                        SDL_DestroyTexture(texture);
-                    }
-                    SDL_DestroySurface(surface);
+                    SDL_FRect dst = { x, y, w, h };
+                    rendered = SDL_RenderTexture(renderer, texture, NULL, &dst);
+                    SDL_DestroyTexture(texture);
                 }
-                SDL_CloseIO(stream);
+                SDL_DestroySurface(surface);
             }
-            if (!rendered) {
-                SDL_RenderDebugText(renderer, x, y, SDL_GetError());
-            }
-            SDL_free(data);
-            return h + 2.0f;
+            SDL_CloseIO(stream);
         }
+        if (!rendered) {
+            SDL_RenderDebugText(renderer, x, y, SDL_GetError());
+        }
+        SDL_free(data);
+        return h + 2.0f;
     }
     return 0.0f;
 }
